# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - SendMessage Action

    @copyright: 2012, 2013, 2014 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.action import ActionBase
from MoinMoin.Page import Page
from MoinMoin.wikiutil import escape, MimeType

from MoinMessage import GPG, MoinMessageError, Message, sendMessage, timestamp, \
                        as_string
from MoinMessageSupport import get_signing_users, get_recipients, get_relays, \
                               get_recipient_details, \
                               MoinMessageRecipientError, OutgoingHTMLFormatter
from MoinSupport import *
from ItemSupport import ItemStore

from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

class SendMessage(ActionBase, ActionSupport):

    "An action that can send a message to another site."

    def get_form_html(self, buttons_html):

        "Present an interface for message sending."

        _ = self._
        request = self.request
        form = self.get_form()

        message = form.get("message", [""])[0]
        recipient = form.get("recipient", [""])[0]
        format = form.get("format", ["wiki"])[0]
        preview = form.get("preview")
        action = form.get("send-action", ["send"])[0]

        # Get a list of potential recipients.

        recipients = get_recipients(request, fetching=False)

        # Prepare the recipients list, selecting the specified recipients.

        recipients_list = []

        if recipients:
            recipients_list += self.get_option_list(recipient, recipients) or []

        recipients_list.sort()

        # Prepare any preview.

        parser_cls = getParserClass(request, format)
        request.formatter.setPage(self.page)
        preview_output = preview and formatText(message, request, request.formatter, inhibit_p=False, parser_cls=parser_cls) or ""

        # Fill in the fields and labels.

        d = {
            "buttons_html"          : buttons_html,
            "format_label"          : escape(_("Message format")),
            "format"                : escattr(format),
            "recipient_label"       : escape(_("Recipient")),
            "recipients_list"       : "\n".join(recipients_list),
            "message_label"         : escape(_("Message text")),
            "message_default"       : escape(message),
            "preview_label"         : escattr(_("Preview message")),
            "preview_output"        : preview_output,
            "send_label"            : escape(_("Send message immediately")),
            "send_selected"         : self._get_selected("send", action),
            "queue_label"           : escape(_("Queue message for sending")),
            "queue_selected"        : self._get_selected("queue", action),
            }

        # Prepare the output HTML.

        html = '''
<table>
    <tr>
        <td class="label"><label>%(recipient_label)s</label></td>
        <td>
            <select name="recipient">
                %(recipients_list)s
            </select>
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(format_label)s</label></td>
        <td>
            <input name="format" type="text" value="%(format)s" size="20" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(message_label)s</label></td>
        <td>
            <textarea name="message" cols="60" rows="10">%(message_default)s</textarea>
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            <input name="preview" type="submit" value="%(preview_label)s" />
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="moinmessage-preview">
%(preview_output)s
        </td>
    </tr>
    <tr>
        <td></td>
        <td>
            <select name="send-action">
                <option value="send" %(send_selected)s>%(send_label)s</option>
                <option value="queue" %(queue_selected)s>%(queue_label)s</option>
            </select>
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>''' % d

        return html

    def do_action(self):

        "Attempt to send the message."

        _ = self._
        request = self.request
        form = self.get_form()

        text = form.get("message", [None])[0]
        recipient = form.get("recipient", [None])[0]
        format = form.get("format", ["wiki"])[0]
        action = form.get("send-action", ["send"])[0]

        queue = action == "queue"

        if not text:
            return 0, _("A message must be given.")

        if not recipient:
            return 0, _("A recipient must be given.")

        homedir = self.get_homedir()
        if not homedir:
            return 0, _("MoinMessage has not been set up: a GPG homedir is not defined.")

        gpg = GPG(homedir)

        # Construct a message from the request.

        message = Message()

        container = MIMEMultipart("related")
        container["Update-Action"] = "store"
        container["To"] = recipient

        # Add the message body and any attachments.

        parser_cls = getParserClass(request, format)

        # Determine whether alternative output types are produced and, if so,
        # bundle them in a multipart/alternative part.

        output_types = getParserOutputTypes(parser_cls)

        if len(output_types) > 1:
            alternatives = MIMEMultipart("alternative")
            container.attach(alternatives)
        else:
            alternatives = container

        # Produce each of the representations.

        for output_type in output_types:

            # HTML must be processed to identify attachments.

            if output_type == "text/html":
                fmt = OutgoingHTMLFormatter(request)
                fmt.setPage(request.page)
                body = formatText(text, request, fmt, inhibit_p=False, parser_cls=parser_cls)
            else:
                body = formatTextForOutputType(text, request, parser_cls, output_type)

            maintype, subtype = output_type.split("/", 1)
            if maintype == "text":
                part = MIMEText(body.encode("utf-8"), subtype, "utf-8")
            else:
                part = MIMEBase(maintype, subtype)
                part.set_payload(body)

            alternatives.attach(part)

        # Produce any identified attachments.

        for pos, (path, filename) in enumerate(fmt.attachments):

            # Obtain the attachment content.

            f = open(path, "rb")
            try:
                body = f.read()
            finally:
                f.close()

            # Determine the attachment type.

            mimetype = MimeType(filename=filename)

            # NOTE: Support a limited set of explicit part types for now.

            if mimetype.major == "image":
                part = MIMEImage(body, mimetype.minor, **mimetype.params)
            elif mimetype.major == "text":
                part = MIMEText(body, mimetype.minor, mimetype.charset, **mimetype.params)
            else:
                part = MIMEApplication(body, mimetype.minor, **mimetype.params)

            # Label the attachment and include it in the message.

            part["Content-ID"] = "attachment%d" % pos
            container.attach(part)

        message.add_update(container)

        # Get the sender details for signing messages.
        # This is not the same as the details for authenticating users in the
        # PostMessage action since the fingerprints refer to public keys.

        signing_users = get_signing_users(request)
        signer = signing_users and signing_users.get(request.user.name)

        # Get the recipient details.

        try:
            parameters = get_recipient_details(request, recipient)
        except MoinMessageRecipientError, exc:
            return 0, exc.message

        type = parameters["type"]
        location = parameters["location"]

        # Obtain the actual location if a relay is specified.

        if parameters["type"] == "relay":
            relays = get_relays(request)
            if not relays:
                return 0, _("No relays are defined for MoinMessage, but one is specified for the recipient.")
            if not relays.has_key(location):
                return 0, _("The relay specified for the recipient is not defined.")

            location = relays[location]

        # Sign, encrypt and send the message.

        message = message.get_payload()

        if not queue and type in ("url", "relay"):
            try:
                if signer:
                    message = gpg.signMessage(message, signer)

                message = gpg.encryptMessage(message, parameters["fingerprint"])

                # Send relayed messages with an extra signature.

                if type == "relay":
                    relaying_user = getattr(self.request.cfg, "moinmessage_gpg_relaying_user")

                    # Signing with the same identity if no special relaying user is
                    # defined.

                    if relaying_user:
                        signer = signing_users and signing_users.get(relaying_user)

                    timestamp(message)
                    message["Update-Action"] = "store"
                    message = gpg.signMessage(message, signer)

                sendMessage(message, location)

            except MoinMessageError, exc:
                return 0, "%s: %s" % (_("The message could not be prepared and sent"), exc)

        # Or queue the message on the specified page.

        elif type == "page":
            page = Page(request, location)
            outbox = ItemStore(page, "messages", "message-locks")
            outbox.append(as_string(message))

        # Or queue the message in a special outbox.

        else:
            outbox = ItemStore(request.page, "outgoing-messages", "outgoing-message-locks")
            outbox.append(as_string(message))

        return 1, _("Message sent!")

    def get_homedir(self):

        "Locate the GPG home directory."

        return getattr(self.request.cfg, "moinmessage_gpg_homedir")

# Action function.

def execute(pagename, request):
    SendMessage(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
